/*********************************************************************/
/* tclparse V2.00 from 14.March 1997                                 */
/* Copyright 1996/97 Stephan Weiss, Heiko Grossmann                  */
/*                   University of Stuttgart, Germany                */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/*********************************************************************/

#ifndef _HANDLER_H_
#define _HANDLER_H_


#include <iostream.h>
#include <fstream.h>


/*######################################################################*/


template<class TYPE>
class CList
{
protected:

  /* Diese Struktur stellt ein Element der Liste dar */
  struct SElement {

    /* Das eigentliche Element */
    TYPE content;

    /* Zeiger auf das vorige bzw. naechste Element */
    SElement *prev;
    SElement *next;
  };

  /* Der Zeiger auf die Wurzel der Liste */
  /* 'root->next' zeigt auf das erste Element */
  /* 'root->prev' zeigt auf das letzte Element */
  SElement *root;

  /* Zeiger fuer die Arbeit mit 'Next()' */
  SElement *internalPtr;

public:

  /* Konstruktor */
  CList( void );

  /* Destruktor */
  ~CList( void );


  /* Einen Zeiger auf den Inhalt des ersten Elements zurueckliefern */
  TYPE *First( void );

  /* Einen Zeiger auf den Inhalt des naechsten Elements zurueckliefern */
  TYPE *Next( void );

  /* Ein Element hinzufuegen */
  void Insert( TYPE &element );

  /* Alle Elemente, die mit 'element' uebereinstimmen, aus der Liste entfernen */
  int Remove( TYPE &element );

  /* Einen Zeiger auf den Inhalt eines bestimmtes Elements zurueckliefern */
  TYPE *Get( TYPE &element );

  /* Abfragen, ob das Element 'element' in der Liste enthalten ist */
  int Contains( TYPE &element );

  /* Abfragen, ob die Liste leer ist */
  int IsEmpty( void );


  CList &operator=( CList &list );
};


/*######################################################################*/


class CTclDataType
{
protected:

  /* Der Name des Befehls */
  char *name;

  /* Die Anzahl der bisherigen Aufrufe des Befehls */
  int calls;


public:

  /* Konstruktoren */
  CTclDataType( void );
  CTclDataType( char *newName, int newCalls=0 );
  CTclDataType( const CTclDataType &dataType );

  /* Destruktor */
  ~CTclDataType( void );


  /* Die Anzahl der bisherigen Aufrufe zurueckliefern */
  int Calls( void );

  /* Den Namen zurueckliefern */
  char *Name( void );


  /* Die Anzahl der bisherigen Aufrufe erhoehen */
  void IncCalls( void );

  /* Die Anzahl der bisherigen Aufrufe neu setzen */
  void SetCalls( int newCalls );

  /* Den Namen neu setzen */
  void SetName( char *newName );


  /* Den Vergleichsoperator ueberladen */
  int operator==( CTclDataType &dataType );

  /* Den Zuweisungsoperator ueberladen */
  CTclDataType &operator=( CTclDataType &dataType );
};
  
  
/*######################################################################*/


class CCommand : public CTclDataType
{
protected:

  /* Die Art des Befehls ( PROCEDURE, WIDGET, IMAGE ) */
  char *commandID;

  /* Bei Prozeduren ist hier Platz fuer die Argumentenliste, bei Bildern
     fuer die Bildart */
  char *list;

  /* Ebenfalls bei Prozeduren ist hier Platz fuer die minimale bzw. maximale
     erlaubte Anzahl an Argumenten */
  int min;
  int max;

public:

  /* Konstruktoren */
  CCommand( void );
  CCommand( char *newName, char *newCommandID, char *newList, int newMin=0, int newMax=0, int newCalls=0 );
  CCommand( const CCommand &command );

  /* Destruktor */
  ~CCommand( void );


  /* Die Befehlsart zurueckliefern */
  char *CommandID( void );

  /* Bei Prozeduren die Argumentenliste zurueckliefern, bei Bildern die Bildart */
  char *List( void );

  /* Bei Prozeduren die Werte 'min' bzw. 'max' zurueckliefern */
  int Max( void );
  int Min( void );


  /* Die Befehlsart neu setzen */
  void SetCommandID( char *newCommandID );

  /* Die Argumentenliste bzw. die Bildart neu setzen */
  void SetList( char *newlist );

  /* Die Werte fuer 'min' bzw. 'max' neu setzen */
  void SetMax( int newMax );
  void SetMin( int newMin );


  /* Den Vergleichsoperator ueberladen */
  int operator==( CCommand &command );

  /* Den Zuweisungsoperator ueberladen */
  CCommand &operator=( CCommand &command );
};

  
/*######################################################################*/


class CVariable : public CTclDataType
{
protected:

  /* Die Umgebung, in der die Variable existiert */
  char *environment;

  /* Der Status der Initialisierung */
  int status;

public:

  /* Konstruktoren */
  CVariable( void );
  CVariable( char *newName, char *newEnvironment, int newStatus=0, int newCalls=0 );
  CVariable( const CVariable &variable );

  /* Destruktor */
  ~CVariable( void );


  /* Den Umgebungsnamen zurueckliefern */
  char *Environment( void );

  /* Den Umgebungsnamen neu setzen */
  void SetEnvironment( char *newEnvironment );

  /* Den Initialisierungsstatus neu setzen */
  void SetStatus( int );

  /* Den Initialisierungsstatus zurueckliefern */
  int Status( void );


  /* Den Vergleichsoperator neu ueberladen */
  int operator==( CVariable &variable);

  /* Den Zuweisungsoperator neu ueberladen */
  CVariable &operator=( CVariable &variable );
};


/*######################################################################*/


class CHandler
{
protected:

  CList<CCommand> commandList;
  CList<CVariable> variableList;
  CList<CCommand> optionList;

public:

  CHandler( void );
  ~CHandler( void );

  /* Einige Praedikate */
  int IsArray( char *name, char *environment );
  int IsCommand( char *name );
  int IsGlobal( char *name );
  int IsImage( char *name );
  int IsOption( char *option, char *command );
  int IsProcedure( char *name );
  int IsVariable( char *name, char *environment );
  int IsWidget( char *name );

  /* Ein bestimmtes Element hinzufuegen */
  void AddCommand( char *name, char *commandID, char *list, int min=0, int max=0, int calls=0 );
  void AddImage( char *name, char *type, int calls=0 );
  void AddOptionList( char *name, char *list );
  void AddProcedure( char *name, char *arguments, int min=0, int max=0, int calls=0 );
  int AddVariable( char *name, char *environment, int status=0, int calls=0 );
  void AddWidget( char *name, char *type, int calls=0 );

  /* Einen Zeiger auf ein bestimmtes Element zurueckliefern */
  CCommand *GetCommand( char *name );
  int GetOptionCount( char *option, char *command );
  CCommand *GetOptionList( char *name );
  CVariable *GetVariable( char *name, char *environment );

  /* Ein bestimmtes Element entfernen */
  int RemoveCommand( char *name );
  int RemoveVariable( char *name, char *environment );

  char *NextEnvironment( char *envString );
  char *ParentWidget( char *name );


  /* Die Informationen der drei Listen in eine Datei schreiben */
  int Write( char *fileName );

  CHandler &operator=( CHandler &handler );
};


#endif


